<?php

namespace Drupal\self_evaluation\Plugin\self_evaluation\ChartType;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\self_evaluation\ChartType\ChartTypeInterface;
use Drupal\self_evaluation\SelfEvaluationAnswerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Abstract Class for ChartType Plugin.
 */
abstract class ChartType extends PluginBase implements ChartTypeInterface, ContainerFactoryPluginInterface {

  /**
   * Entity type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The module extension list.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleList;

  /**
   * Config Factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Class constructor.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, ModuleHandlerInterface $module_handler, ModuleExtensionList $module_list, ConfigFactoryInterface $configFactory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->moduleHandler = $module_handler;
    $this->moduleList = $module_list;
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): ChartType {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('extension.list.module'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function description() {
    return $this->pluginDefinition['description'];
  }

  /**
   * Build Options for chart building.
   *
   * @param array $override_options
   *   An array with overriding values options.
   *
   * @return array
   *   Chart pptions.
   */
  protected function getOptions(array $override_options = []): array {
    $settings = $this->configFactory->get('charts.settings');
    $chart_settings = $settings->get('charts_default_settings');
    $chart_settings = array_merge($chart_settings, $override_options);

    return [
      'type' => '',
      'title' => '',
      'xaxis_title' => '',
      'yaxis_title' => '',
      'yaxis_min' => '',
      'yaxis_max' => '',
      'three_dimensional' => FALSE,
      'title_position' => $chart_settings['title_position'] ?: 'out',
      'legend_position' => $chart_settings['legend_position'] ?: 'top',
      'legend' => TRUE,
      'background' => 'transparent',
      'data_labels' => $chart_settings['data_labels'] ?? TRUE,
      'tooltips' => $chart_settings['tooltips'] ?? TRUE,
      'colors' => $chart_settings['colors'] ?? [],
      'min' => $chart_settings['min'] ?? 0,
      'max' => $chart_settings['max'] ?? 100,
      'yaxis_prefix' => $chart_settings['yaxis_prefix'] ?? '',
      'yaxis_suffix' => $chart_settings['yaxis_suffix'] ?? '',
      'data_markers' => $chart_settings['data_markers'] ?? FALSE,
      'red_from' => $chart_settings['red_from'] ?? 0,
      'red_to' => $chart_settings['red_to'] ?? 50,
      'yellow_from' => $chart_settings['yellow_from'] ?? 50,
      'yellow_to' => $chart_settings['yellow_to'] ?? 85,
      'green_from' => $chart_settings['green_from'] ?? 85,
      'green_to' => $chart_settings['green_to'] ?? 100,
      'width' => $chart_settings['width'] ?? 100,
      'width_units' => $chart_settings['width_units'] ?? '%',
      'height' => $chart_settings['height'] ?? 50,
      'height_units' => $chart_settings['height_units'] ?? '%',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildChart(array $data, SelfEvaluationAnswerInterface $self_evaluation_answer): array {
    if (!$this->moduleHandler->moduleExists('charts')) {
      return [];
    }

    $settings = $this->configFactory->get('charts.settings');
    $chart_settings = $settings->get('charts_default_settings');
    $library = $chart_settings['library'] ?? 'chartjs';
    // @todo make colors configurable.
    $colors = ['#76ddfb', '#b0b0b0'];
    $options = $this->getOptions(['colors' => $colors]);

    $labels = !empty($data['labels']) ? $data['labels'] : [];
    $datasets = [];
    $build = [];
    // @todo make legends configurable.
    $dataset_legends = [
      $this->t('Your score'),
      $this->t('Average score'),
    ];

    $info = $this->moduleList->getExtensionInfo('charts');
    if (preg_match('#^8\.x\-3.*#', $info['version'])) {
      if (!empty($data['datasets'])) {
        foreach ($data['datasets'] as $index => $dataset) {
          $datasets[] = [
            'name' => $this->t('@index', ['@index' => $dataset_legends[$index]]),
            'color' => $colors[$index],
            'type' => '',
            'data' => $dataset,
          ];
        }
      }

      $build = [
        '#theme' => 'charts_blocks',
        '#library' => (string) $library,
        '#categories' => $labels,
        '#seriesData' => $datasets,
        '#secondaryOptions' => [],
        '#options' => $options,
        '#override' => [],
      ];
    }

    if (preg_match('#^5\..*#', $info['version'])) {
      if (!empty($data['datasets'])) {
        foreach ($data['datasets'] as $index => $dataset) {
          $datasets[] = [
            '#type' => 'chart_data',
            '#data' => $dataset,
            '#color' => $colors[$index],
            '#title' => $this->t('@index', ['@index' => $dataset_legends[$index]]),
            '#decimal_count' => 2,
          ];
        }
      }

      $build = [
        '#type' => 'chart',
        '#chart_type' => $options['type'],
        '#chart_library' => (string) $library,
        '#title' => $options['title'],
        '#title_position' => $options['title_position'],
        '#tooltips' => $options['tooltips'],
        '#data_labels' => $options['data_labels'],
        '#colors' => $options['colors'],
        '#background' => $options['background'],
        '#legend' => $options['legend'],
        '#legend_position' => $options['legend_position'],
        'xaxis' => [
          '#type' => 'chart_xaxis',
          '#title' => '',
          '#labels' => $labels,
        ],
      ] + $datasets;
    }

    return $build;
  }

}
